%Messlistener

%Shaker-Regelung mit Sinus are free software.
%you can redistribute it and/or under terms of the GNU General Public
%License as published by the Free Software Foundation.
%Shaker-Regelung mit Sinus are distributed in the hope that will useful,
%but without any warranty, without even the implied warranty of
%merchantability or fitness for a particular purpose. See the GNU for more
%details:
%Florian Fltzinger
%Hochschule fr angewandte Wissenschaften Mnchen
%University of applied sciences Munich

%In dem "Messlistener" werden die Signale fr Beschleunigung und Kraft
%gemessen und der Weg berechnet. Auerdem werden die FFT Spektren zur
%Bestimmung der real gemessenen Amplitudenwerte sowie der realen Frequenz
%(Ist-Werte)erstellt. Es wird ein Messfenster erstellt, um das Anschwingen
%des Shakers zu bercksichtigen. Alle Signale werden geplottet. Die
%Hauftfunktion ist hier die Regelschleife, welche am Ende die Regelung
%durchfhrt. Die Funktion "Erg_speichern" wird hier im Regelendekriterium
%aufgerufen.

function Messlistener(src,event)


global  Kraft Beschl Weg f V fmax  b tb F tF w tw a da df fs  GE Ergebnis m Messwerte  Ausgangssignal ;

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    %Plotten der Messdaten (unberarbeitet)
    figure(6);
    subplot(2,2,1);
    plot(event.TimeStamps, event.Data)
    grid on;
    grid minor;
    xlabel('Zeit in [s]');
    ylabel('Beschleuniung u. Kraft in [V]');
    title('gemessene Beschleunigung und Kraft');
    %legend('Beschleunigung','Kraft');
    
    %Signalumrechnungen Kalibrierung 
    Be=(event.Data(:,1))./(10.48*10*0.001);             %Kalibrierung des Beschleunigungssensors
    Kr=(event.Data(:,2))./(11.241*100*0.001);           %Kalibrierung des Kraftsensors
      
    %Plotten der Messdaten mit physik. Einheiten   
    subplot(2,2,2);
    plot(event.TimeStamps, Be, event.TimeStamps, Kr);
    grid on;
    grid minor;
    xlabel('Zeit in [s]');
    ylabel('Beschleuniung u. Kraft in [m/s] bzw. [N]');
    title('errechnete Beschleunigung und Kraft');
    %legend('Beschleunigung','Kraft');
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    %Schwingungsfenster zum Messen herausschneiden
    Be1=Be((2*(fs)/5):((fs)-(2*(fs)/5)));               %Messfenster fr weitere Berechnungen ohne Einschwingen
    Kr1=Kr((2*(fs)/5):((fs)-(2*(fs)/5)));               %Messfenster fr weitere Berechnungen ohne Einschwingen
    
    subplot(2,2,3:4);
    plot(event.TimeStamps((2*(fs)/5):((fs)-(2*(fs)/5))), Be1, event.TimeStamps((2*(fs)/5):((fs)-(2*(fs)/5))), Kr1);   %Messfenster 0.2s fr weitere Berechnungen ohne Einschwingen,
    grid on;                                                                                                          %da minimale Frequenz =5Hz =>Eine Periode im Fenster
    grid Minor;
    xlabel('Zeit in [s]');
    ylabel('Beschleuniung u. Kraft in [m/s] bzw. [N]');
    title('Messfenster Beschleunigung und Kraft');
    %legend('Beschleunigung','Kraft');
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%                                                                                                                  
                                                                                                                      
    %Messwerte abspeichern
    ErgZeit=event.TimeStamps((2*(fs)/5):((fs)-(2*(fs)/5)));                  %Speichern der Zeitwerte vom Messfenster
    ErgBeschl=Be1;                                                           %Speichern der Beschleunigungswerte vom Messfenster 
    ErgKraft=Kr1;                                                            %Speichern der Kraftwerte vom Messfenster
    Messerg=[ErgZeit,ErgBeschl,ErgKraft];                                    %Zeit- Beschleunigung- und Kraftwerte in Matrix speichern
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    % FFT 
    blocksize = fs/10;                                                       %berlappung der FFT mit Blockgre 100000
    noverlap = ceil(blocksize*2/3);                                          %berlappung der FFT mit ceil Funktion und 2/3 der Blockgre
    window = hann(blocksize)*blocksize/sum(hann(blocksize));                 %Fensterfunktion der FFT hann normiert auf die Blockgre => Amplitudenwert
         
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    [fft_Be,fr,t,p] = spectrogram(Be1,window,noverlap,fs);                   %Spektrogram der Beschleunigung
    [fft_Kr,fr,t,p] = spectrogram(Kr1,window,noverlap,fs);                   %Spektrogram der Kraft
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
     
    V1=abs(fft_Be)*2;
    Beschl=max(V1(:,1))/blocksize;                      %Berrechnung der Ist-Amplitude Beschleunigung und zugehrigen Frequenzwert
    
    [y,frequenz_berechnet]=max((V1(:,1))/blocksize);    %Berechnung der realen Frequenz um sicher zu stellen, dass 
    freal=frequenz_berechnet-1;                         %die gemessene Frequenz gleich der Sollfrequenz ist.
    disp(freal);
    
    V2=abs(fft_Kr)*2;
    Kraft=max(V2(:,1))/blocksize;                       %Berrechnung der Ist-Amplitude Kraft

    Weg=Beschl/(2*pi*f)^2;                              %Berrechnung der Ist-Amplitude Weg
    
    figure (7);
    plot(fr,(V1(:,1))/blocksize,fr,(V2(:,1))/blocksize);                        %Plotten der FFT
    grid on;
    grid minor;
    xlabel('Frequenz');
    ylabel('Amplitudenwerte Beschleuniung u. Kraft in [m/s] bzw. [N]');
    title('Frequenzspektrum');
    %legend('Beschleunigung','Kraft');
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
 
    %Regelendekriterium
    if(f>fmax)                                          %Beenden der Regelung wenn f>fmax
     disp('Maximalfrequenz erreicht');                  %Ausgabe Maximalfrequenz erreicht
     src.stop()                                         %Stoppen 
     Ergebnis(:,4)=Ergebnis(:,4).*1000000;              %Weg in Mikrometer 
     [ Messwerte ] = Erg_speichern();                   %Aufruf der Funktion "Erg_speichern", um alle Messwerte in eine Struktur zu speichern
    end
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    %Regelschleife
    
    switch V                                                                                %Fallunterscheidung Beschleunigung, Weg, Kraft

    case 1                                                                                  %Fall: Beschleunigung
        
        
            if ((b-tb/2) < Beschl) && (Beschl < (b+tb/2)) && (f<=fmax) && (f==freal)        %reale Frequenz (Amplitude) wird mit der Sollfrequenz (Amplitude) verglichen
               
               GE{m,1}=f;                                                                   %geregelte Frequenzen in Gesamtergebnis (Zeitsignale) speichern
               GE{m,2}=Messerg;                                                             %geregelte Beschleunigungsmesswerte (Zeitsignale) in Gesamtergebnis abspeichern
               Ergebnis(m,:)=[f,Beschl,Kraft,Weg];                                          %alle Zeitsignale mit zugehrigen Frequenzen in Ergebnis speichern                        
               m=m+1;                                                                       %Variable i weiter zhlen, damit nchste Zeile in Ergerbnis mit nchster Frequenz belegt wird
               
               
               disp('Messen');
               disp('bei');disp(f);disp('Hz');
               f=f+df;                                                                      %Frequenz um Frequenzintervall df weiter rauf zhlen
               
                
            else
                if ((b-tb/2) > Beschl)&&(f<=fmax)                                           %wenn Beschleunigung kleiner als untere Toleranzgrenze und Maximalfrequenz noch nicht erreicht
                  
                  a=a+da;                                                                   %Amplitude um Amplitudenschritt da vergrern
                  disp(a);
                   
                  
                elseif (Beschl > (b+tb/2))&&(f<=fmax)                                       %wenn Beschleunigung grer als obere Toleranzgrenze und Maximalfrequenz noch nicht erreicht
                  
                  a=a-da;                                                                   %Amplitude um Amplitudenschritt da verkleinern
                  disp(a);
                    
                  
               end
               
            end
        
      
    case 2                                                                                  %Fall: Weg
        
        
            if ((w-tw/2) < Weg) && (Weg < (w+tw/2)) && (f<=fmax) && (f==freal)              %reale Frequenz (Amplitude) wird mit der Sollfrequenz (Amplitude) verglichen
               
               GE{m,1}=f;                                                                   %geregelte Frequenzen in Gesamtergebnis (Zeitsignale) speichern
               GE{m,2}=Messerg;                                                             %geregelte Wegmesswerte (Zeitsignale) in Gesamtergebnis abspeichern
               Ergebnis(m,:)=[f,Beschl,Kraft,Weg];                                          %alle Zeitsignale mit zugehrigen Frequenzen in Ergebnis speichern
               m=m+1;                                                                       %Variable i weiter zhlen, damit nchste Zeile in Ergerbnis mit nchster Frequenz belegt wird
               
               
               disp('Messen');
               disp('bei');disp(f);disp('Hz');
               f=f+df;                                                                      %Frequenz um Frequenzintervall df weiter rauf zhlen

            
               else  
                    if ((w-tw/2) > Weg)&&(f<=fmax)                                          %wenn Weg kleiner als untere Toleranzgrenze und Maximalfrequenz noch nicht erreicht
                  
                    a=a+da;                                                                 %Amplitude um Amplitudenschritt da vergrern
                    disp(a);

                  
                    elseif (Weg > (w+tw/2))&&(f<=fmax)                                      %wenn Weg grer als obere Toleranzgrenze und Maximalfrequenz noch nicht erreicht
                  
                    a=a-da;                                                                 %Amplitude um Amplitudenschritt da verkleinern
                    disp(a);

                    end
               
            end
        
        
    case 3                                                                                  %Fall: Kraft
        
        
            if ((F-tF/2) < Kraft)&&(Kraft < (F+tF/2)) && (f<=fmax) && (f==freal)            %reale Frequenz (Amplitude) wird mit der Sollfrequenz (Amplitude) verglichen
               
               GE{m,1}=f;                                                                   %geregelte Frequenzen in Gesamtergebnis (Zeitsignale) speichern
               GE{m,2}=Messerg;                                                             %geregelte Kraftmesswerte (Zeitsignale) in Gesamtergebnis abspeichern
               Ergebnis(m,:)=[f,Beschl,Kraft,Weg];                                          %alle Zeitsignale mit zugehrigen Frequenzen in Ergebnis speichern
               m=m+1;                                                                       %Variable i weiter zhlen, damit nchste Zeile in Ergerbnis mit nchster Frequenz belegt wird
               
               
               disp('Messen');
               disp('bei');disp(f);disp('Hz');
               f=f+df;                                                                      %Frequenz um Frequenzintervall df weiter rauf zhlen
               

            else
                 if ((F-tF/2) > Kraft)&&(f<=fmax)                                           %wenn Kraft kleiner als untere Toleranzgrenze und Maximalfrequenz noch nicht erreicht
                  
                  a=a+da;                                                                   %Amplitude um Amplitudenschritt da vergrern
                  disp(a);  
                   
                  
                 elseif (Kraft > (F+tF/2))&&(f<=fmax)                                       %wenn Kraft grer als obere Toleranzgrenze und Maximalfrequenz noch nicht erreicht
                  
                  a=a-da;                                                                   %Amplitude um Amplitudenschritt da verkleinern
                  disp(a);

                 end
             
            end
            
        
    end

 
    

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
end

    
    




