%setup

%Shaker-Regelung mit Sinus are free software.
%you can redistribute it and/or under terms of the GNU General Public
%License as published by the Free Software Foundation.
%Shaker-Regelung mit Sinus are distributed in the hope that will useful,
%but without any warranty, without even the implied warranty of
%merchantability or fitness for a particular purpose. See the GNU for more
%details:
%Florian Fltzinger
%Hochschule fr angewandte Wissenschaften Mnchen
%University of applied sciences Munich

%Die Funktion "setup" fungiert als Bindeglied zwischen Initialisierung der
%Messgerte sowie Aufruf der Listenerfunktionen. In ihr wird auerdem 
%Speicherplatz fr Ergebnisse geschaffen und die Abtastrate festgelegt.

function [s,lh2,lh1] = setup()

global  df   fs  f a fmax Ausgangssignal m GE Ergebnis ;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Speicherplatz/ort fr Ergebnis
Ergebnis=zeros(((fmax-f)/df)+1,4);                          %Stellt ein Cell-Array bereit fr die Regelergebnisse (Beschl, Kraft, Weg)
m=1;                                                        %Initialisierung der Variable i
GE = cell(((fmax -f)/df)+1,2);                              %Stellt ein Cell-Array bereit fr die Zeitsignale der Einzelnen Frequenzen
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

s = daq.createSession('ni');                                %Initialisierung des Messgerts
s.DurationInSeconds = df;


addAnalogOutputChannel(s,'PXI1Slot2', 'ao0', 'Voltage');    %Ausgabekanal

s.IsContinuous = true;                                      %Kontinuierliche Ausganbe

fs=100000;                                                  %Abtastrate in scans/sec.
s.Rate = fs;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Funktion zur Erzeugung eines Ausgangssignals
%Initialisierung des Ausgabesignals
Zeit=1;                                                     %Siganlzeit 1s
n=Zeit*fs;                                                  %Anzahl Signalwerte
t=(1:n)/fs;                                                 %Zeitwerte
Ausgangs=a*sin(2*pi*f*t);                                   %Ausgangssignal Sinus mit einstellbarer Amplitude und Frequenz
Ausgangssignal=Ausgangs';                                   %Ausgangssignal transponiert



lh1 = addlistener(s,'DataRequired',@(src,event)Ausgabelistener(src,event));  %Listener lh1 zur Ausgabe des Signals (Funktionsaufruf Ausgabelistener)


queueOutputData(s,Ausgangssignal);                          %Ausgabesignal aneinanderreihen


  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


addAnalogInputChannel(s,'PXI1Slot2', 'ai2', 'Voltage');     %Einganskanal Beschleunigung
addAnalogInputChannel(s,'PXI1Slot2', 'ai3', 'Voltage');     %Einganskanal Kraft


    

Aufrufsampels                          = fs*Zeit ;          %Anzahl der Werte in einem Aufruf ist Abtastrate * Zeit Ausgabesignal
s.NotifyWhenDataAvailableExceeds       = Aufrufsampels ;    %Aufrufrate des Listeners lh2 
lh2 = addlistener(s,'DataAvailable', @(scr,event)Messlistener(scr,event));    %Listener lh2 zum Messen und Regeln (Funktionsaufruf Messlistener lh2)  


startBackground(s);                                         %Start der Background-Session

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

end

